import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.*;
import javax.sql.*;
import com.reportmill.base.*;
import com.reportmill.graphics.*;
import com.reportmill.text.*;
import com.reportmill.shape.*;
import snap.util.*;

/*
 * This servlet generates an AdHoc report for a list of movies, allowing user to select Attributes, Groupings,
 * Sortings, output and Table of Contents
 */
public class AdHoc extends HttpServlet {

    // A unique integer
    int _req;

/**
 * Implements standard servlet get to build template and run/return report.
 */
public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
{
    // Handle Attribute selection - each session has an Attrs int array of attribute indexes
    if(request.getParameter("Attr") != null) {
        int attrs[] = (int[])request.getSession().getAttribute("Attrs"); if(attrs==null) attrs = new int[0];
        int index = StringUtils.intValue(request.getParameter("Attr"));
        if(index==99) attrs = null;
        else { int len = attrs.length; attrs = Arrays.copyOf(attrs,len+1); attrs[len] = index; }
        request.getSession().setAttribute("Attrs", attrs);
    }
    
    // Handle Grouping selection - each session has a Groups int array of attribute indexes
    if(request.getParameter("Group") != null) {
        int groups[] = (int[])request.getSession().getAttribute("Groups"); if(groups==null) groups = new int[0];
        int index = StringUtils.intValue(request.getParameter("Group"));
        if(index==99) groups = null;
        else { int len = groups.length; groups = Arrays.copyOf(groups,len+1); groups[len] = index; }
        request.getSession().setAttribute("Groups", groups);
    }
    
    // Handle sort selection
    if(request.getParameter("Sort") !=null) {
        int sort = StringUtils.intValue(request.getParameter("Sort"));
        request.getSession().setAttribute("Sort", new Integer(sort));
    }
    
    // Handle special Output options (PDF and Flash)
    String output = (String)request.getSession().getAttribute("Out"); if(output==null) output = "html";
    if(request.getParameter("Output")!=null) {
        String newOut = request.getParameter("Output");
        if(newOut.equals("Flash")) {
            output = output.equals("html")? "Flash" : "html";
            request.getSession().setAttribute("Out", output);
        }
        else output = newOut;
    }
    
    // Handle Resource requests
    if(request.getParameter("RES")!=null) {
        
        // Get resources map, imageRoot and resourceKey
        Map map = (Map)request.getSession().getAttribute("Resources"); if(map==null) map = new Hashtable();
        String imageRoot = "./AdHoc?RES=";
        String key = (String)request.getParameter("RES");
        String resourceKey = imageRoot + key;
        
        // If key is Template, get bytes and add to resource map
        if(key.equals("Template")) {
            byte b[] = getTemplate(request).getBytes();
            map.put(resourceKey, b);
        }
        
        // Get resourceKey and resource bytes and load into response
        byte bytes[] = (byte[])map.get(resourceKey); map.remove(resourceKey);
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
        return;
    }

    // Generate HTML with customized template from getTemplate() method
    RMDocument template = getTemplate(request);
    RMDocument report = template.generateReport(_hollywood, output.equals("PDF"));
    
    // If output equals PDF, generate and return
    if(output.equals("PDF")) {
        byte bytes[] = report.pdfBytes();
        response.setContentType("application/pdf");
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
        return;
    }
    
    // If output equals Flash, generate and add Resources map to session
    /*if(output.equals("Flash")) {
        // Generate Flash and load into a session "Resources" Map
        byte flashBytes[] = report.getBytesFlash();
        String imageRoot = "./AdHoc?RES=FlashReport" + (_req++) + ".swf";
        Map map = RMMapUtils.newMap(imageRoot, flashBytes);
        request.getSession().setAttribute("Resources", map);
        
        // Generate container HTML file and load into session
        String html = com.reportmill.output.RMHTMLWriter.htmlString(report, imageRoot, "Report");
        html = StringUtils.replace(html, "<center>", ""); // Strip the center tag that RM adds
        request.getSession().setAttribute("html", html);
    }*/
    
    // If output equals html, generate HTML Map (htmlBytes + resources) and add Resources map to session
    if(output.equals("html")) {
        //String imageRoot = "./AdHoc?RES=";
        //Map map = report.getMapHTML(imageRoot);
        //request.getSession().setAttribute("Resources", map);
    
        // Get htmlBytes and load into session
        byte bytes[] = report.getBytesHTML(); //(byte[])map.get("htmlBytes"); map.remove("htmlBytes");
        String html = StringUtils.getISOLatinString(bytes);
        request.getSession().setAttribute("html", html);
    }

    // Forward AdHoc.jsp
    try {
        getServletConfig().getServletContext().getRequestDispatcher("/AdHoc.jsp").forward(request, response);   
    } catch (Exception e) { e.printStackTrace(); }
}

/*
 * The getTemplate() method loads a basic template and configures it based of selected attributes ("Attrs" in the
 * session) and selected groupings ("Groups" in the session).
 */
RMDocument getTemplate(HttpServletRequest request)
{
    // Load basic template and get its table
    RMDocument template = new RMDocument(getServletContext().getResourceAsStream("AdHoc.rpt"));
    RMTable table = (RMTable)template.getPage(0).getChildWithClass(RMTable.class); // or use getChildWithName()
    
    // Get default table row and add/get Header row
    RMTableRow row = (RMTableRow)table.getChildWithTitle("Objects Details");
    RMTableRow headerRow = table.addHeader("Objects");

    // Get attribute indexes and set # of columns for row and headerRow
    int attrs[] = (int[])request.getSession().getAttribute("Attrs"); if(attrs==null) return template;
    row.setNumberOfColumns(attrs.length);
    headerRow.setNumberOfColumns(attrs.length);
    
    // Iterate over attrs indexes and configure row and headerRow columns
    for(int i=0, iMax=attrs.length; i<iMax; i++) {
        AHAttribute attr = (AHAttribute)_attributes.get(attrs[i]);
        attr.configure(row.getColumn(i), false);
        attr.configure(headerRow.getColumn(i), true);
        
        // Apply sort request URL to header columns 
        headerRow.getColumn(i).setURL("./AdHoc?Sort=" + attrs[i]);
    }
    
    // Call arrange on row/headerRow to resize columns for relative widths (and set headerRow color & font)
    row.layout(); headerRow.layout();
    headerRow.setColor(new RMColor(.9f));
    headerRow.setFont(RMFont.getFont("Arial Bold", 10));
    
    // Get groups indexes and add groupings
    int groups[] = (int[])request.getSession().getAttribute("Groups");
    RMColor colors[] = { new RMColor(1,.7f,.7f), new RMColor(.7f,.7f,1), new RMColor(1,.7f,1) };
    for(int i=0, iMax=groups==null? 0 : groups.length; i<iMax; i++) {
    
        // Get group attribute and add/get groupRow for group key
        AHAttribute group = getAttribute(groups[i]);
        RMTableRow groupRow = table.addDetails(group.key);
        
        // Set number of columns to 2 and configure
        groupRow.setNumberOfColumns(2);
        groupRow.getColumn(0).setText("@" + group.key + "@");
        getAttribute(8).configure(groupRow.getColumn(1), false);
        groupRow.getColumn(1).getXString().addChars("Total Rev: ", 0);
        
        // Set row font (large for each level of grouping) and color (from list above)
        groupRow.setFont(RMFont.getFont("Arial Bold", 14 + i*4));
        groupRow.setColor(colors[i]);
    }
    
    // Get "Sort" attribute from session
    Number sortNum = (Number)request.getSession().getAttribute("Sort");
    int sortIndex = sortNum==null? 0 : sortNum.intValue();
    AHAttribute sortAttr = getAttribute(sortIndex);
    
    // Get bottom level grouping and add Sort for sort attribute
    RMGrouping grouping = table.getGrouping("Objects");
    RMSort sort = new RMSort(sortAttr.key, sortAttr.format==null? RMSort.ORDER_ASCEND : RMSort.ORDER_DESCEND);
    grouping.addSort(sort);
    grouping.addSort("title"); // Might as well back all sorts up by title
    
    // Set table's listKey to "Movies"
    table.setDatasetKey("Movies");
    template.layout();
    return template;
}

// Define a static list of report attributes (name, key, align, format, relative width)
static AHAttribute getAttribute(int anIndex) { return (AHAttribute)_attributes.get(anIndex); }
static List  _attributes = new ArrayList();
static {
    _attributes.add(new AHAttribute("Title", "getTitle", 0, null, 2));
    _attributes.add(new AHAttribute("Category", "getCategory", 0, null, 1));
    _attributes.add(new AHAttribute("Rating", "getRating", 2, null, .5f));
    _attributes.add(new AHAttribute("Date Released", "getShowDate", 0, null, 1));
    RMNumberFormat money = new RMNumberFormat("$ #,##0;-$ #,##0");
    RMNumberFormat percent = new RMNumberFormat("0.00%;-0.00%");
    _attributes.add(new AHAttribute("Revenue", "getRevenue", 1, money, 1));
    _attributes.add(new AHAttribute("% Total", "getRevenue/Up.total.getRevenue*100", 1, percent, .5f));
    _attributes.add(new AHAttribute("Running Total", "Running.total.getRevenue", 1, money, 1));
    _attributes.add(new AHAttribute("Studio", "getStudio.getName", 0, null, 2));
    _attributes.add(new AHAttribute("Group Total", "total.getRevenue", 1, money, -1));
}

// Define a class to represent report attributes
static class AHAttribute {
    String       name;
    String       key;
    byte         align;
    RMFormat     format;
    float        width;
    public AHAttribute(String n, String k, int a, RMFormat f, float w) {
        name = n; key = k; align = (byte)a; format = f; width = w; }
    public void configure(RMTextShape aText, boolean useName) {
        aText.setText(useName? name : ("@" + key + "@"));
        //aText.setAlign(align);
        if(format!=null) aText.setFormat(format);
        if(width>0) aText.setWidth(width);
    }
}

// Load copy of hollywood database from RM resource
static Map _hollywood = new RMXMLReader().readObject(RMXMLReader.class.getResource("/com/reportmill/examples/HollywoodDB.xml"));

}