package rm;
import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.reportmill.base.*;
import com.reportmill.shape.*;
import snap.util.*;

public class Catalog extends HttpServlet {

    // The catalog template
    static RMDocument      _template;
    
    // The table of contents template
    static RMDocument      _tableOfContentsTemplate;

/**
 * Implements standard servlet Get method to handle request and return catalog.
 */
public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
{
    // Get list of products for reqeust
    List products = Product.products(request);
    
    // Create shopping cart list
    List cart = new Vector();

    // Add product selections to cart
    String s[] = request.getParameterValues("products");
    for(int i=0; i<s.length; i++)
        cart.add(products.get(StringUtils.intValue(s[i])));
        
    // Randomize order of products
    cart = Product.randomize(cart);

    // Handle PDF
    if(request.getParameter("pdf")!=null) {
    
        // Write out catalog XML
        //new RMXMLWriter().writeObject(cart, "/tmp/Catalog.xml");
        
        // Generate catalog report
        RMDocument report1 = getTemplate().generateReport(cart);
        
        // Generate catalog table of contents
        RMTableOfContents toc = new RMTableOfContents(report1);
        
        // Generate catalog table of contents report
        RMDocument report2 = getTableOfContentsTemplate().generateReport(toc);
        
        // Add catalog report to table of contents report
        report2.addPages(report1);
        
        // Get PDF bytes and add to response
        byte bytes[] = report2.getBytesPDF();
        response.setContentType("application/pdf");
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
    }

    // forward Movies.jsp
    else try {
        getServletConfig().getServletContext().getRequestDispatcher("/Catalog.jsp").forward(request, response);   
    } catch (Exception e) { e.printStackTrace(); }
}

/** Returns the catalog template. */
RMDocument getTemplate()
{
    if(_template==null)
        _template = new RMDocument(inputStream("Catalog.rpt"));
    return _template;
}

/** Returns the table of contents template. */
RMDocument getTableOfContentsTemplate()
{
    if(_tableOfContentsTemplate==null)
        _tableOfContentsTemplate = new RMDocument(inputStream("CatalogTOC.rpt"));
    return _tableOfContentsTemplate;
}

/** Returns an input stream for the named servlet resource. */
InputStream inputStream(String aName) { return getServletContext().getResourceAsStream(aName); }

public static String productsHTML(HttpServletRequest request)
{
    List products = Product.products(request);
    StringBuffer html = new StringBuffer();
    
    for(int i=0; i<products.size(); i++) {
        Product p = (Product)products.get(i);
        html.append("<OPTION value=\"" + i + "\">" + p.productName + "</OPTION>\n");
    }
    return html.toString();
}

}