import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.reportmill.base.*;
import com.reportmill.shape.*;

/**
 * A simple servlet showing drill down reports.
 */
public class Drilldown extends HttpServlet {

    // The list of studio objects
    static List studios;
    
    // This list of movie objects
    static List movies;
    
    // The list of actor objects
    static List talents;
    
    // The list of movie rol objects
    static List movieRoles;
    
    // A random number generator
    static Random random = new Random();

/** Creates a new drill down servlet instance. */
public Drilldown()
{
    // Fetch our data (once since it's read-only) we're just using our built-in example data,
    // in real life this is where you might hit your persistence layer or other datasource
    if(studios==null) {
        Map hollywood = new RMXMLReader().readObject(RMXMLReader.class.getResource("/com/reportmill/examples/HollywoodDB.xml"));
        studios = (List)hollywood.get("Studios");
        movies = (List)hollywood.get("Movies");
        talents = (List)hollywood.get("Talent");
        movieRoles = (List)hollywood.get("MovieRoles");
    }
}

/**
 * An implementation of the standard servlet get method.
 */
public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
{
    // get a different report based on the parameters passed to the script
    String resource = request.getParameter("resource");
    String reportType = request.getParameter("type");
    
    // Handle a resource request
    if(resource!=null) {
        ReportData resourceData = (ReportData)request.getSession().getAttribute(resource);
        sendBytesToResponse(resourceData.bytes, response, "application/x-shockwave-flash");
    }
    
    // Handle Movie report
    else if("movies".equals(reportType))
        getMoviesReport(request, response);

    // Handle cast report
    else if("cast".equals(reportType))
        getCastReport(request, response);

    // Handle talent report
    else if("talent".equals(reportType))
        getTalentReport(request, response);

    // Handle Studio Report
    else getStudiosReport(request, response);
}

public void getMoviesReport(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException
{
    // Get studio key from request
    String studioName = request.getParameter("studio");
    
    //  If studio name is null, we can't continue so we return the default report
    if(studioName == null)
        getStudiosReport(request, response);

    // If studio is non-null, get the matching studio
    else {
    
        // We'll use a convenience method in com.reportmill.foundation to get the actual studio object
        Map studio = null;
        
        // Iterate over studios to find studio matching the name
        for(int i=0; i<studios.size() && studio==null; i++)
            if(RMUtils.equals(RMKey.getValue(studios.get(i), "getName"), studioName))
                studio = (Map)studios.get(i);

        // Create a map of "userInfo" that contains the information needed to drilldown from this report
        Map userInfo = new Hashtable();
        userInfo.put("baseURL", baseURL());
        String pdfURL = pdfBaseURL(request) + "?type=movies&studio=" + studio.get("getName") + "&format=PDF";
        userInfo.put("pdfURL", pdfURL);
        String reportFormat = request.getParameter("format");
        reportFormat = (reportFormat != null) ? reportFormat : "HTML";
        userInfo.put("format", reportFormat);
        userInfo.put("session", request.getSession());

        // now we can generate the report 
        RMDocument doc = new RMDocument(getServletContext().getResourceAsStream("templates/MoviesDrilldown.rpt"));
        RMDocument report = doc.generateReport(studio, userInfo, false);

        if(reportFormat.equals("PDF"))
            returnPDFData(request, response, report);
        else returnHTMLData(request, response, report);
    }
}

public void getCastReport(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException
{
    // Get movie key from request
    String movieName = request.getParameter("movie");
    
    // If null, we can't continue so we return the default report
    if(movieName == null)
        getStudiosReport(request, response);

    // Otherwise find matching movie name
    else {
    
        // We'll use a convenience method in com.reportmill.foundation to get the actual movie object
        Map movie = null;
        
        // Iterate over movies to find matching movie
        for(int i=0; i<movies.size() && movie==null; i++)
            if(RMUtils.equals(RMKey.getValue(movies.get(i), "getTitle"), movieName))
                movie = (Map)movies.get(i);

        // Create a map of "userInfo" that contains the information needed to drilldown from this report
        Map userInfo = new Hashtable();
        userInfo.put("baseURL", baseURL());
        String pdfURL = pdfBaseURL(request) + "?type=cast&movie=" + movie.get("getTitle") + "&format=PDF";
        userInfo.put("pdfURL", pdfURL);
        String reportFormat = request.getParameter("format");
        reportFormat = (reportFormat != null) ? reportFormat : "HTML";
        userInfo.put("format", reportFormat);
        userInfo.put("session", request.getSession());

        // Now we can generate the report 
        RMDocument doc = new RMDocument(getServletContext().getResourceAsStream("templates/CastDrilldown.rpt"));
        RMDocument report = doc.generateReport(movie, userInfo, false);

        if(reportFormat.equals("PDF"))
            returnPDFData(request, response, report);
        else returnHTMLData(request, response, report);
    }
}

public void getTalentReport(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException
{
    // Get movie key from request
    String talentName = request.getParameter("lastName");
    
    // If null, we can't continue so we return the default report
    if(talentName == null)
        getStudiosReport(request, response);

    // Otherwise find matching talent name
    else {
    
        // we'll use a convenience method in com.reportmill.foundation to get the actual movie object
        Map talent = null;
        
        // Iterate over talents to find matching last name
        for(int i=0; i<talents.size() && talent==null; i++)
            if(RMUtils.equals(RMKey.getValue(talents.get(i), "getLastName"), talentName))
                talent = (Map)talents.get(i);

        // Create a map of "userInfo" that contains the information needed to drilldown from this report
        Map userInfo = new Hashtable();
        userInfo.put("baseURL", baseURL());
        String pdfURL = pdfBaseURL(request) + "?type=talent&lastName=" + talent.get("getLastName") + "&format=PDF";
        userInfo.put("pdfURL", pdfURL);
        String reportFormat = request.getParameter("format");
        reportFormat = (reportFormat != null) ? reportFormat : "HTML";
        userInfo.put("format", reportFormat);
        userInfo.put("session", request.getSession());

        // now we can generate the report 
        RMDocument doc = new RMDocument(getServletContext().getResourceAsStream("templates/TalentMoviesDrilldown.rpt"));
        RMDocument report = doc.generateReport(talent, userInfo, false);

        if(reportFormat.equals("PDF"))
            returnPDFData(request, response, report);
        else returnHTMLData(request, response, report);
    }
}

public void getStudiosReport(HttpServletRequest request, HttpServletResponse response)
    throws IOException, ServletException
{
    // create a map of "userInfo" that contains the information needed to drilldown from this report
    Map userInfo = new Hashtable();
    userInfo.put("baseURL", baseURL());
    userInfo.put("pdfURL", pdfBaseURL(request) + "?format=PDF");
    String reportFormat = request.getParameter("format");
    reportFormat = (reportFormat != null) ? reportFormat : "HTML";
    userInfo.put("format", reportFormat);
    userInfo.put("session", request.getSession());

    RMDocument doc = new RMDocument(getServletContext().getResourceAsStream("templates/StudiosDrilldown.rpt"));
    RMDocument report = doc.generateReport(studios, userInfo, false);

    if(reportFormat.equals("PDF"))
        returnPDFData(request, response, report);
    else returnHTMLData(request, response, report);
}

// generic method for returning HTML version of generated report
public void returnHTMLData(HttpServletRequest request, HttpServletResponse response, RMDocument report)
    throws IOException, ServletException
{
    byte[] bytes = report.getBytesHTML();
    sendBytesToResponse(bytes, response, "text/html");
}

// generic method for returning PDF version of generated report
public void returnPDFData(HttpServletRequest request, HttpServletResponse response, RMDocument report)
    throws IOException, ServletException
{
    byte[] bytes = report.pdfBytes();
    sendBytesToResponse(bytes, response, "application/pdf");
}

// generic method for sending bytes back to a response with a particular MIME type
public void sendBytesToResponse(byte[] bytes, HttpServletResponse response, String mimeType)
    throws IOException, ServletException
{
    response.setContentType(mimeType);
    response.setContentLength(bytes.length);
    ServletOutputStream out = response.getOutputStream();
    out.write(bytes);
    out.close();
}

// this is a simple inner class used as an object wrapper around the report data
protected class ReportData {
    byte[] bytes;
    public ReportData(byte[] dataBytes) { bytes = dataBytes; }
}

public String baseURL()
{
    return "./Drilldown";
}

public String pdfBaseURL(HttpServletRequest aRequest)
{
    return aRequest.getRequestURL().toString();
}

}