import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.reportmill.base.*;
import com.reportmill.shape.*;

public class Form1040EZ extends HttpServlet {

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
    {
        // Create a Map for taxpayer info
        Map taxpayerInfo = new Hashtable();
        
        // Iterate over request parameters and add to taxpayer
        Enumeration e = request.getParameterNames();
        while (e.hasMoreElements()) {
            String key = (String)e.nextElement();
            taxpayerInfo.put(key, request.getParameter(key));
        }
        
        // Some hashtable values need type conversion
        // We'll also pre-calculate some of the values rather than leave
        // everything to the template. We will however leave a few
        // things out so that this example simulates reporting from
        // custom business logic as well as deriving values directly
        // in the template itself.
        // We should really do some proper error reporting here, but
        // for now we'll just fail silently if the number conversion raises any exceptions.
        try {
            float wages = Float.parseFloat((String)taxpayerInfo.get("wages"));
            float taxWithheld = Float.parseFloat((String)taxpayerInfo.get("taxWithheld"));
            float interest = Float.parseFloat((String)taxpayerInfo.get("interest"));
            taxpayerInfo.put("wages", new Float(wages));
            taxpayerInfo.put("taxWithheld", new Float(taxWithheld));
            taxpayerInfo.put("interest", new Float(interest));
            taxpayerInfo.put("grossIncome", new Float(wages+interest));
            float taxableIncome = (wages+interest-7450 > 0) ? (wages+interest-7450) : 0.0f;
            taxpayerInfo.put("taxableIncome",new Float(taxableIncome));
            taxpayerInfo.put("tax", new Float(taxableIncome*0.28));
        } catch (Exception exception) {}

        // have ReportMill generate the report
        RMDocument template = new RMDocument(getServletContext().getResourceAsStream("1040EZ.rpt")); 
        RMDocument report = template.generateReport(null,taxpayerInfo,false);
        byte[] pdfBytes = report.pdfBytes();

        // send the pdf bytes to the response
        response.setContentType("application/pdf");
        response.setContentLength(pdfBytes.length);
        ServletOutputStream out = response.getOutputStream();
        out.write(pdfBytes);
        out.close();
    }
}


