import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.reportmill.base.*;
import com.reportmill.shape.*;

/**
 * A simple servlet to let the user select a list of movies by category and return it
 * as a report in HTML, PDF, Flash or CSV.
 */
public class Movies extends HttpServlet {

    // The template stored as a static so it can be loaded once and shared
    static RMDocument _template;
    
/**
 * An implementation of the basic servlet Get method.
 */
public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
{
    // Handle "HTML Report" submit button
    if(request.getParameter("HTML") != null) {
    
        // Get objects and generate report
        Object objects = request.getSession().getAttribute("data");
        RMDocument report = getTemplate().generateReport(objects, false);
        
        // Generate HTML Map (htmlBytes + resources) and add "Resources" map to session
        //String imageRoot = "./Movies?RES=";
        //Map map = report.getMapHTML(imageRoot);
        //request.getSession().setAttribute("Resources", map);
        
        // Get htmlBytes and load into response
        byte bytes[] = report.getBytesHTML(); //(byte[])map.get("htmlBytes"); map.remove("htmlBytes");
        response.setContentType("text/html");
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
    } else

    // Handle "PDF Report" submit button
    if(request.getParameter("PDF") != null) {
    
        // Get objects and generate report
        Object objects = request.getSession().getAttribute("data");
        RMDocument report = getTemplate().generateReport(objects);
        
        // Generate PDF bytes and load into response
        byte bytes[] = report.pdfBytes();
        response.setContentType("application/pdf");
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
    }
    
    // Handle "CSV Report" submit button
    else if(request.getParameter("CSV") != null) {
    
        // Get objects and generate report
        Object objects = request.getSession().getAttribute("data");
        RMDocument report = getTemplate().generateReport(objects, false);
        
        // Generate CSV bytes and load into response
        byte bytes[] = report.getBytesCSV();
        response.setContentType("application/vnd.ms-excel");
        response.setHeader("Content-Disposition", "attachment;filename=Report.csv");
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
    }
    
    // Handle Resource request
    else if(request.getParameter("RES")!=null) {
    
        // Get resources map and imageRoot
        Map map = (Map)request.getSession().getAttribute("Resources");
        String imageRoot = "./Movies?RES=";
        
        // Get resourceKey and resource bytes and load into response
        String resourceKey = imageRoot + (String)request.getParameter("RES");
        byte bytes[] = (byte[])map.get(resourceKey); map.remove(resourceKey);
        response.setContentLength(bytes.length);
        response.getOutputStream().write(bytes);
        response.getOutputStream().close();
    }

    else fetchData(request, response);
}

// This method fetches selected movies for user and put them in session    
public void fetchData(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException 
{
    // Fetch built-in RM movie data - In real-life, this is where you'd fetch objects from db
    Map hollywood = new RMXMLReader().readObject(RMXMLReader.class.getResource("/com/reportmill/examples/HollywoodDB.xml"));
    List movies = (List)hollywood.get("Movies"), filtered = movies;

    // If Category is not "All", get movies
    String filter = (String)request.getParameter("CATEGORY");
    if(!filter.equals("All")) {
        filtered = new Vector();
        for(int i=0; i<movies.size(); i++) {
            Map h = (Map)movies.get(i);
            String c = (String)h.get("getCategory");
            if(c.equals(filter))
                filtered.add(h);
        }
    }

    // Put filtered movies in session
    request.getSession().setAttribute("data", filtered);
    
    // forward Movies.jsp
    try {
        getServletConfig().getServletContext().getRequestDispatcher("/Movies.jsp").forward(request, response);   
    } catch (Exception e) { e.printStackTrace(); }
}

/** Returns the shared template loaded into the static variable. */
public RMDocument getTemplate()
{
    if(_template==null)
        _template = new RMDocument(getServletContext().getResourceAsStream("Movies.rpt"));
    return _template;
}

}