package rm;
import java.io.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.reportmill.base.*;
import com.reportmill.shape.*;
import snap.util.*;

public class QuoteGen extends HttpServlet {

    // The quote template
    static RMDocument      _template;
    
    // A number formatter
    static DecimalFormat   _moneyFormatter = new DecimalFormat("$ #,##0.00");

/**
 * Standard servlet get implmentation.
 */
public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
{
    // Get shopping cart from session
    ShoppingCart cart = getShoppingCart(request);
    
    // Update field values
    cart._name = getStringOrNull(request.getParameter("name"));
    cart._email = getStringOrNull(request.getParameter("email"));
    cart._company = getStringOrNull(request.getParameter("company"));
    cart._address1 = getStringOrNull(request.getParameter("addr1"));
    cart._address2 = getStringOrNull(request.getParameter("addr2"));
    cart._cityStateZip = getStringOrNull(request.getParameter("cityStateZip"));
    cart._phone = getStringOrNull(request.getParameter("phone"));
    cart._fax = getStringOrNull(request.getParameter("fax"));
    
    // Handle add developer app
    if(request.getParameter("addDev")!=null)
        cart._developer = Math.max(cart._developer + StringUtils.intValue(request.getParameter("devQty")), 0);
    
    // Handle remove developer app
    if(request.getParameter("remDev")!=null)
        cart._developer = Math.max(cart._developer - StringUtils.intValue(request.getParameter("devQty")), 0);
        
    // Handle add deployment cpu
    if(request.getParameter("addDep")!=null)
        cart._deployment = Math.max(cart._deployment + StringUtils.intValue(request.getParameter("depQty")), 0);

    // Handle remove deployment cpu
    if(request.getParameter("remDep")!=null)
        cart._deployment = Math.max(cart._deployment - StringUtils.intValue(request.getParameter("depQty")), 0);
        
    // Handle add support
    if(request.getParameter("addSup")!=null)
        cart._support = Math.max(cart._support+1, 0);
        
    // Handle remove support
    if(request.getParameter("remSup")!=null)
        cart._support = Math.max(cart._support-1, 0);
        
    // Handle add upgrade
    if(request.getParameter("addUpg")!=null)
        cart._upgrades = Math.max(cart._upgrades+1, 0);
        
    // Handle remove upgrade
    if(request.getParameter("remUpg")!=null)
        cart._upgrades = Math.max(cart._upgrades-1, 0);
        
    // Handle add training
    if(request.getParameter("addTrn")!=null)
        cart._training = Math.max(cart._training+1, 2);
        
    // Handle remove traning
    if(request.getParameter("remTrn")!=null)
        cart._training = cart._training==2? 0 : Math.max(cart._training-1, 0);

    // Handle PDF generation
    if(request.getParameter("pdf")!=null)
        generatePDF(request, response);
        
    // Handle flash generation
    else if(request.getParameter("flash")!=null)
        generatePDF(request, response);

    // forward QuoteGen.jsp
    else try {
        getServletConfig().getServletContext().getRequestDispatcher("/QuoteGen.jsp").forward(request, response);   
    } catch (Exception e) { e.printStackTrace(); }
}

/**
 * Generate PDF.
 */
public void generatePDF(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException 
{
    ShoppingCart cart = getShoppingCart(request);
    RMDocument report = getTemplate().generateReport(cart);
    byte[] bytes = report.pdfBytes();
    returnBytes(request, response, bytes, "application/pdf");
}

/**
 * Append PDF bytes to response.
 */
public void returnBytes(HttpServletRequest request, HttpServletResponse response, byte[] bytes, String type) throws IOException, ServletException 
{
    response.setContentType(type);
    response.setContentLength(bytes.length);
    ServletOutputStream out = response.getOutputStream();
    out.write(bytes);
    out.close();
}

/**
 * Returns the quote template (loading it the first time).
 */
RMDocument getTemplate()
{
    // If template hasn't been loaded, load it
    if(_template==null)
        _template = new RMDocument(getServletContext().getResourceAsStream("Quote.rpt"));
        
    // Return template
    return _template;
}

/**
 * Returns the session shopping cart.
 */
public static ShoppingCart getShoppingCart(HttpServletRequest request)
{
    // Get shopping cart from session
    ShoppingCart cart = (ShoppingCart)request.getSession().getAttribute("ShoppingCart");
    
    // If no cart for session, create one and add to session
    if(cart==null) {
        cart = new ShoppingCart();
        request.getSession().setAttribute("ShoppingCart", cart);
    }
    
    // Return cart
    return cart;
}

public static String name(HttpServletRequest request) { return getString(getShoppingCart(request)._name); }

public static String email(HttpServletRequest request) { return getString(getShoppingCart(request)._email); }

public static String company(HttpServletRequest request) { return getString(getShoppingCart(request)._company); }

public static String address1(HttpServletRequest request) { return getString(getShoppingCart(request)._address1); }

public static String address2(HttpServletRequest request) { return getString(getShoppingCart(request)._address2); }

public static String cityStateZip(HttpServletRequest request) { return getString(getShoppingCart(request)._cityStateZip); }

public static String phone(HttpServletRequest request) { return getString(getShoppingCart(request)._phone); }

public static String fax(HttpServletRequest request) { return getString(getShoppingCart(request)._fax); }

public static String order(HttpServletRequest request)
{
    ShoppingCart cart = getShoppingCart(request);
    String order = cart.order();
    return order.length()>0? order : "< None >";
}

public static String totalPrice(HttpServletRequest request)
{
    ShoppingCart cart = getShoppingCart(request);
    return _moneyFormatter.format(cart.totalPrice());
}

/**
 * Converts a null string to empty string.
 */
static String getString(String aString) { return aString==null? "" : aString; }

/**
 * Converts an empty string to null string.
 */
static String getStringOrNull(String aString) { return aString.length()>0? aString : null; }

}