package rm;
import java.util.*;

/**
 * Represents a shopping cart.
 */
public class ShoppingCart {

    // The customer name
    String      _name;
    
    // The customer email
    String      _email;
    
    // The customer company
    String      _company;
    
    // The customer address 1
    String      _address1;
    
    // The customer address 2
    String      _address2;
    
    // The customer city, state zip
    String      _cityStateZip;
    
    // The customer phone
    String      _phone;
    
    // The customer fax
    String      _fax;
    
    // The number of developer app licenses
    int         _developer;
    
    // The number of deployment cpu licenses
    int         _deployment;
    
    // The number of support contracts
    int         _support;
    
    // The number of upgrades
    int         _upgrades;
    
    // The number of tranings
    int         _training;

/**
 * Returns the name.
 */
public String getCustName()  { return _name; }

/**
 * Returns the email.
 */
public String getEmail()  { return _email; }
    
/**
 * Returns the company.
 */
public String getCompany()  { return _company; }
    
/**
 * Returns the address1.
 */
public String getAddress1()  { return _address1; }
 
/**
 * Returns the address2.
 */
public String getAddress2()  { return _address2; }

/**
 * Returns the cityStateZip.
 */
public String getCityStateZip()  { return _cityStateZip; }

/**
 * Returns the phone.
 */
public String getPhone()  { return _phone; }
    
/**
 * Returns the fax.
 */
public String getFax()  { return _fax; }
    
/**
 * Returns the developer.
 */
public String developer()
{
    return _developer>0 && _upgrades<5? _developer + " " + developerName() + " (" +
        QuoteGen._moneyFormatter.format(developerPrice()) + ")<br>" : "";
}

/**
 * Returns the deveoper name.
 */
public String developerName()
{
    return _developer>1? "Layout Applications" : "Layout Application";
}

public String deployment()
{
    return _deployment>0 && _upgrades<5? _deployment + " " + deploymentName() + " (" +
        QuoteGen._moneyFormatter.format(deploymentPrice()) +")<br>" : "";
}

public String deploymentName()
{
    String name = "Single Processor Deployment License";
    if(_deployment>15) name += " - volume 16 or more";
    else if(_deployment>7) name += " - volume 8 or more";
    else if(_deployment>3) name += " - volume 4 or more";
    else if(_deployment>1) name += " - volume 2 or more";
    return name;
}

public String support()
{
    return _support>0? _support + " Support (20 incidents, 1 year) (" +
        QuoteGen._moneyFormatter.format(supportPrice()) + ")<br>" : "";
}

public String upgrades()
{
    return _upgrades>0? _upgrades%4 + " Upgrade Subscription (1 year, 15% of software sales) (" +
        QuoteGen._moneyFormatter.format(upgradesPrice()) + ")<br>" : "";
}

public String training()
{
    return _training>1? "On-site training (" + _training + " days) (" +
        QuoteGen._moneyFormatter.format(trainingPrice()) + ")<br>" : "";
}

int developerPrice()
{
    return _developer*295;
}

int deploymentPrice()
{
    return _deployment*unitDeploymentPrice();
}

int unitDeploymentPrice()
{
    return _deployment>=16? 2995 : _deployment>=8? 3495 : _deployment>=4? 3995 : _deployment>=2? 4495 : 4995;
}

int supportPrice()
{
    return _support*1495;
}

int softwarePrice()
{
    return developerPrice() + deploymentPrice();
}

float upgradesPrice()
{
    return (_upgrades%4)*softwarePrice()*.15f;
}

float trainingPrice()
{
    return _training>1? 5995 + (_training-2)*1500 : 0;
}

float totalPrice()
{
    return developerPrice() + deploymentPrice() + supportPrice() + upgradesPrice() + trainingPrice();
}

String totalPriceString()
{
    return "$ " + totalPrice();
}

public Vector lineItems()
{
    // Create line items list
    Vector lineItems = new Vector();
    
    // Add developer line item
    if(_developer>0 && _upgrades<5)
        lineItems.add(lineItem("Layout Application", 1000, _developer, 295));
        
    // Add deployment line item
    if(_deployment>0 && _upgrades<5)
        lineItems.add(lineItem(deploymentName(), 2000, _deployment, unitDeploymentPrice()));
        
    // Add support line item
    if(_support>0)
        lineItems.add(lineItem("Support (20 incidents, 1 year)", 3000, _support, 1495));
        
    // Add upgrades line item
    if(_upgrades>0)
        lineItems.add(lineItem("Upgrade Subscription (1 Year, 15% of software sales)", 4000,
            _upgrades%4, upgradesPrice()/(_upgrades%4)));
            
    // Add training line item
    if(_training>1)
        lineItems.add(lineItem("On-site Training (" + _training + " days)", 5000,
            _training, trainingPrice()/_training));
    
    // Add spacer line items
    for(int i=lineItems.size(); i<24; i++)
        lineItems.add(new Hashtable());
    
    // Return line items
    return lineItems;
}

Hashtable lineItem(String name, int partNumber, int quantity, float unitPrice)
{
    Hashtable li = new Hashtable(4);
    li.put("name", name);
    li.put("partNumber", new Integer(partNumber));
    li.put("quantity", new Integer(quantity));
    li.put("unitPrice", new Float(unitPrice));
    return li;
}

public String order()
{
    return developer() + deployment() + support() + upgrades() + training();
}

}